# nodc-archive-parser

Python code for parsing NODC archive MULDARS data formats.

This project, funded by the North Pacific Research Board, includes Python code for parsing [Principal MULti-Discipline Archives Retrieval System (MULDARS)](https://www.nodc.noaa.gov/access/dataformats.html) data formats archived at the National Centers for Environmental Information (NCEI), formerly National Oceanic Data Center (NODC).

## Scope

The project was funded for and is currently limited to these specific archive formats:

* [F030](https://www.nodc.noaa.gov/General/NODC-Archive/f030.html) - Inter-tidal Organisms and Habitats
* [F124](https://www.nodc.noaa.gov/General/NODC-Archive/f124.html) - Zooplankton
* [F132](https://www.nodc.noaa.gov/General/NODC-Archive/f132.html) - Benthic Organisms

## Prerequisites

This code was initially developed with Python 2.7.12 and the following freely available Python packages:

* requests 2.18.14
* pandas 0.18.1

Pandas makes use of the numpy library; the version of numpy installed on the original developer’s system was 1.9.3.  

## Methodology

The archive format is a column-based text file that contains multiple record types such as stations and sample results, with each record defining its own columns.  Code for equipment, weather, and more are used throughout the files, as are NODC taxonomic code numbers for biological data.  

The parsing methodology involves reading record format files which define how
each record type should be parsed. Code lookup tables and taxonomic code lookup tables are
also parsed and used to interpret the archived data file. A single data file is
split into a separate comma-separated values (CSV) file for each record type.
Code descriptions are inserted into the output to provide an explanation for
each code value, and the Integrated Taxonomic Information System (ITIS) can be
queried to find current Taxonomic Serial Numbers and scientific names for the
now-deprecated NODC taxonomic codes.  The code can also be used to join
information from one table to another, for example to add latitude and longitude
coordinates to sample results so that sample data can be plotted on a map.

## How To Use This Project

For example input and output data, see the **data** folder.

See **demonstration.py** for an example of how to use the parsing code. The demonstration steps through parsing example files for the three formats mentioned above.  

In general, the steps to using the code are:

1. Import the **nodcparse** module.
2. Specify the location of the **TAXNUM.DAT** file containing NODC taxonomic codes, preferably version 8 since that version includes ITIS TSNs.
3. Specify the location of a **code table file** that has definitions for all code tables used by the target record format.
4. Specify the location of a **folder with CSV files describing each record in the record format**. Optionally, joins.csv and join_by_sequence.csv files may also be included in this folder.  See the examples for the structure of these files.
5. Specify the location of the **archived data file** to process.
6. Specify the **output folder location**. A CSV file for each record type will be written to the folder. 
7. Run the **main** function in the module. Or, if you wish to programmatically create the various code lookups and record formats rather than read them from a file, you may call **extract_to_csv** directly.
8. Indicate whether or not the **ITIS Web service** should be queried to obtain modern TSNs and scientific names for taxa.  Because querying ITIS can take several minutes, we recommended not using ITIS (`use_itis=False`) the first time through just to make sure the parsing code is working. If the parsing code works, then run it again with the `use_itis` parameter set to `True`.

The module also includes a **split_file** function for simply splitting the archived text file into separate files for each record type. The files will have the same column based structure as the original. This is useful for visually inspecting your data before running the actual parsing code. Format errors can be easier to spot when the column definitions are the same for the entire file.
